# TODO: This custom green screens need work, refactor and 
# make them work better and simplify te code.
# Consider this part as experimental.
# Was working in the first stage but it is not clear how 
# to handle it in code (x.x)


import requests
import os
import time
from yta_general_utils.file_downloader import download_image
from yta_general_utils.image_processor import resize_without_scaling
from dotenv import load_dotenv

load_dotenv()

MODEL_LEONARDO_DIFFUSION_XL = '1e60896f-3c26-4296-8ecc-53e2afecc132'

def __generate_image(prompt):
    """
    Makes a request to generate an image. It returns the generation id.

    This method needs the non-free API working.
    """
    #return '69503ef1-85c8-41cc-b292-f672946920d7'
    url = "https://cloud.leonardo.ai/api/rest/v1/generations"

    payload = {
        'modelId': MODEL_LEONARDO_DIFFUSION_XL,
        'width': 1024,   # test 1536
        'height': 576,   # test 864
        'num_images': 1,
        'prompt': prompt
    }

    headers = {
        "accept": "application/json",
        "authorization": "Bearer " + str(os.getenv('LEONARDOAI_API_KEY')),
        "content-type": "application/json",
    }

    response = requests.post(url, json = payload, headers = headers)
    response = response.json()

    print(response)

    return response['sdGenerationJob']['generationId']

def __download_generated_image(generation_id, output_filename):
    """
    Downloads the AI-generated image by using the provided 'generation_id'.

    It downloads the image and resizes it to 1920x1080.
    """
    url = 'https://cloud.leonardo.ai/api/rest/v1/generations/' + str(generation_id)

    headers = {
        "accept": "application/json",
        "authorization": "Bearer " + str(os.getenv('LEONARDOAI_API_KEY')),
    }

    response = requests.get(url, headers = headers)
    response = response.json()

    # TODO: Do a passive waiting
    is_downloadable = True

    if len(response['generations_by_pk']['generated_images']) == 0:
        is_downloadable = False

    while not is_downloadable:
        time.sleep(10)
        print('Doing a request in loop')

        # We do the call again
        response = requests.get(url, headers = headers)
        response = response.json()
        
        if len(response['generations_by_pk']['generated_images']) > 0:
            is_downloadable = True

    downloadable_url = response['generations_by_pk']['generated_images'][0]['url']

    download_image(downloadable_url, output_filename)
    resize_without_scaling(output_filename)

def generate_image(prompt, output_filename):
    """
    Generates an AI image with the provided 'prompt' and stores it locally
    as the 'output_filename'.
    """
    generation_id = __generate_image(prompt)
    __download_generated_image(generation_id, output_filename)




from objects.customgreenscreenvideo import CustomGreenScreenVideo
from yta_image_edition_module.green_screen import get_green_screen_position
from os import getenv as os_getenv, listdir as os_listdir
from random import choice as random_choice
from utils.general import create_tmp_filename
from moviepy.editor import VideoFileClip

class CustomVideo:
    __TOUSE_ABSOLUTE_PATH = os_getenv('TOUSE_ABSOLUTE_PATH')

    def __init__(self, video_filename):
        # We generate a temp frame to get green screen position information
        tmp_frame_filename = create_tmp_filename('tmp_frame.png')
        VideoFileClip(video_filename).save_frame(tmp_frame_filename, t = 0)

        green_screen_info = get_green_screen_position(tmp_frame_filename)
        rgb_color = green_screen_info['rgb_color']
        self.ulx = green_screen_info['ulx']
        self.uly = green_screen_info['uly']
        self.drx = green_screen_info['drx']
        self.dry = green_screen_info['dry']

        gs_width = self.drx - self.ulx
        gs_height = self.dry - self.uly
        aspect_ratio = gs_width / gs_height
        if aspect_ratio > (1920 / 1080):
            # We keep the width, but we calculate the height to keep 16/9 aspect ratio
            gs_height_for_16_9 = gs_width * (1080 / 1920)
            # Now we know the new height, so we only have to center the video on 'y' axis
            difference = gs_height_for_16_9 - gs_height
            # My video (16/9) will not fit the whole width
            self.uly -= difference / 2
            self.dry += difference / 2
        elif aspect_ratio < (1920 / 1080):
            # We keep the height, but we calculate the width to keep 16/9 aspect ratio
            gs_width_for_16_9 = gs_height * (1920 / 1080)
            # Now we know the new width, so we only have to center the video on 'x' axis
            difference = gs_width_for_16_9 - gs_width
            self.ulx -= difference / 2
            self.drx += difference / 2
        # TODO: Are we sure that those coords are int and make a real 16/9 with int numbers?

        self.cgsv = CustomGreenScreenVideo(video_filename, rgb_color, self.ulx, self.uly, self.drx, self.dry, None, None, None, None, None, None, None, None, None, None)

    def save(self, output_filename):
        self.cgsv.save(output_filename)

    def insert_video(self, video_filename, output_filename):
        self.cgsv.insert_video(video_filename, output_filename)

    def from_clip(self, clip):
        """
        Receives a 'clip' and generates a new one that is the provided one inside of the
        green screen.
        """
        return self.cgsv.from_clip(clip)

    def insert_image(self, image_filename, output_filename):
        self.cgsv.insert_image(image_filename, output_filename)


from PIL import Image

def image_file_has_transparency(image_filename: str):
    """
    Checks if the provided 'image_filename' has transparency or not.
    This method returns True if yes or False if not.
    """
    if not image_filename:
        return None
    
    if not image_file_is_valid(image_filename):
        return None
    
    return image_has_transparency(Image.open(image_filename))

def image_has_transparency(image: Image):
    """
    Checks if the provided image (read with pillow) has transparency.
    This method returns True if yes or False if not.
    """
    if image.info.get("transparency", None) is not None:
        return True
    if image.mode == "P":
        transparent = image.info.get("transparency", -1)
        for _, index in image.getcolors():
            if index == transparent:
                return True
    elif image.mode == "RGBA":
        extrema = image.getextrema()
        if extrema[3][0] < 255:
            return True

    return False

def rgb_to_hex(r, g, b):
    """
    Returns the provided RGB color as a hex color.
    """
    return '#{:02x}{:02x}{:02x}'.format(r, g, b)

def image_file_is_valid(image_filename: str):
    """
    Tries to open the 'image_filename' provided to check if it is corrupt or it is valid. It 
    returns True if the provided image is valid, or False if is corrupt.

    # TODO: Maybe move this to 'yta-general-utils' (?)
    """
    if not image_filename:
        return None
    
    if not image_file_is_valid(image_filename):
        return None

    try:
        im = Image.open(image_filename)
        im.verify()
        im.close()
    except (IOError, OSError, Image.DecompressionBombError) as e:
        return False
    
    return True